<?php

namespace forge12\woocommerce\accessories {
    if (!defined('ABSPATH')) {
        exit;
    }

    require_once('Shortcode.class.php');

    /**
     * Class Requiredaccessories
     * Responsible to handle the frontend of the accessories
     *
     * @package forge12\woocommerce\accessories
     */
    class RequiredAccessories
    {
        private static $instance = null;

        /**
         * List of Shortcodes
         */
        private $listShortcodes = array();

        public static function getInstance()
        {
            if (self::$instance == null) {
                self::$instance = new self;
            }
            return self::$instance;
        }

        /**
         * Admin constructor.
         */
        public function __construct()
        {
            $this->listShortcodes[] = new Shortcode();
            $Settings = UIRequiredAccessories::getSettings();

            add_shortcode('f12-wc-accessories-required', [$this, 'doShortcode']);

            // Check if the settings are enabled
            if ($Settings['enable'] == 1) {
                add_action('init', array($this, 'init'));

                // show the required products
                add_action('woocommerce_after_single_product_summary', array($this, 'render'), $Settings['priority']);

                // show the header defined in the settings
                add_filter('f12_woo_accessories_required_products_heading', array($this, 'addSettingsTitle'), 10, 2);

                // show the description defined in the settings
                add_filter('f12_woo_accessories_required_products_description', array($this, 'addSettingsDescription'), 10, 2);

                // add all required products to the cart whenever the parent product is added
                add_action('woocommerce_add_to_cart', array($this, 'addToCart'), 10, 6);

                // hide the remove cart item link in the cart
                add_filter('woocommerce_cart_item_remove_link', array($this, 'disableRemoveFromCartButton'), 10, 2);

                // ensure required accessories are not removed as long as the parent element is not removed
                add_filter('woocommerce_stock_amount_cart_item', array($this, 'disableRemoveCartOnUpdate'), 10, 2);

                // add an extra description about the accessoire for the parent product
                add_action('woocommerce_after_cart_item_name', array($this, 'addCartDescription'), 10, 2);

                // remove accessories if the parent is removed
                add_action('woocommerce_cart_item_removed', array($this, 'removeAccessoriesRelatedToRemovedProduct'), 10, 2);

                // add additional information below excerpt
                add_action('woocommerce_single_product_summary', array($this, 'showHintBelowExcerpt'), 21);

                // Manipulate the product price if required accessories are available
                add_filter('woocommerce_get_price_html', array($this, 'maybeAdjustPrice'), 10, 2);

                // Show the product hint title
                add_filter('f12_woo_accessories_required_product_hint_title', array($this, 'addSettingsHintTitle'));

                // Text displayed within the cart
                add_filter('f12_woo_accessories_required_product_cart_title', array($this, 'addSettingsCartTitle'));

                // add filter
                add_filter('f12_woo_accessories_cart', array($this, 'getAccessoriesByProductID'), 12, 3);
            }
        }

        /**
         * Get all required accessories for the given product id
         * @param array $ids
         * @param int $product_id
         * @return array
         */
        public function getAccessoriesByProductID($ids, $product_id, $excluded = array())
        {
            $product_ids = accessories::getAccessoireIDs($product_id, 'required');
            $product_ids = array_diff($product_ids, $excluded);
            return array_merge($ids, $product_ids);
        }

        /**
         * Replace the default cart title with the settings cart title
         * @param string $content
         * @return mixed
         */
        public function addSettingsCartTitle($content)
        {
            $settings = UIRequiredAccessories::getSettings();
            if (!empty($settings['product_cart_title'])) {
                $content = $settings['product_cart_title'];
            }
            return $content;
        }

        /**
         * Replace the default title with the settings hint title
         * @param string $content
         * @return mixed
         */
        public function addSettingsHintTitle($content)
        {
            $settings = UIRequiredAccessories::getSettings();
            if (!empty($settings['product_hint_title'])) {
                $content = $settings['product_hint_title'];
            }
            return $content;
        }

        /**
         * Adjust the product price
         * @param string $price
         * @param \WC_Product $product
         */
        public function maybeAdjustPrice($price, $product)
        {
            $settings = UIRequiredAccessories::getSettings();
            if ($settings['price_summary'] != 1) {
                return $price;
            }

            $IDs = accessories::getAccessoireIDs($product->get_id(), 'required');

            if (!empty($IDs)) {
                $newPrice = $product->get_regular_price();
                foreach ($IDs as $id) {
                    $AccessoireProduct = wc_get_product($id);
                    if ($AccessoireProduct) {
                        $newPrice = (double)$AccessoireProduct->get_price() + (double)$newPrice;
                    }
                }

                $price = wc_price($newPrice) . $product->get_price_suffix();

            }

            return $price;
        }

        /**
         * Show an list of all required products below the excerpt.
         */
        public function showHintBelowExcerpt()
        {
            $settings = UIRequiredAccessories::getSettings();
            if ($settings['product_hint'] != 1) {
                return;
            }

            $IDs = accessories::getAccessoireIDs(get_the_ID(), 'required');

            $listOfProducts = array();
            foreach ($IDs as $id) {
                $Product = wc_get_product($id);

                if ($Product) {
                    $listOfProducts[] = '<a href="' . esc_url($Product->get_permalink()) . '" target="_blank" rel="noopener noreferrer" title="' . esc_attr($Product->get_title()) . '">' . esc_html($Product->get_title()) . '</a>';
                }
            }
            if(empty($listOfProducts)){
                return;
            }
            ?>
            <div class="accessories-required-list">
                <strong>
                    <?php _e(apply_filters('f12_woo_accessories_required_product_hint_title', 'Contains the following accessories:'), 'f12-woo-accessories'); ?>
                </strong>
                <p>
                    <?php echo implode(',', $listOfProducts); ?>
                </p>
            </div>
            <?php
        }


        /**
         * Remove all items that are assigned to the remove button via the accessoire id
         * @param string $cart_item_key
         * @param \WC_Cart $cart
         */
        public function removeAccessoriesRelatedToRemovedProduct($cart_item_key, $cart)
        {
            $items = $cart->get_cart();

            foreach ($items as $cart_item) {
                if (isset($cart_item['_accessoire_parent_key']) && $cart_item['_accessoire_parent_key'] == $cart_item_key) {
                    // remove also all accessories related to the removed product
                    $cart->remove_cart_item($cart_item['key']);
                }
            }
        }

        /**
         *
         * @param $cart_item
         * @param $cart_item_key
         */
        public function addCartDescription($cart_item, $cart_item_key)
        {
            if (isset($cart_item['_accessoire_parent_key'])) {
                if (isset($cart_item['_accessoire_is_required']) && $cart_item['_accessoire_is_required'] == true) {
                    $parent_cart_item = WC()->cart->get_cart_item($cart_item['_accessoire_parent_key']);
                    /**
                     * @var $product \WC_Product
                     */
                    $product = $parent_cart_item['data'];
                    ?>
                    <div class="accessoire-description">
                        <strong><?php _e(apply_filters('f12_woo_accessories_required_product_cart_title', 'Part of: '), 'f12-woo-accessories'); ?></strong> <?php esc_html_e($product->get_title()); ?>
                    </div>
                    <?php
                }
            }
        }

        /**
         * Ensure that required products are not removed by reducing the amount of items added to the cart.
         * @param int $amount
         * @param string $cart_item_key
         * @return int
         */
        public function disableRemoveCartOnUpdate($amount, $cart_item_key)
        {
            $Settings = UIRequiredAccessories::getSettings();

            if ($Settings['lock'] == 1) {
                $cart_item = WC()->cart->get_cart_item($cart_item_key);

                if (isset($cart_item['_accessoire_parent_key'])) {
                    if (isset($cart_item['_accessoire_is_required']) && $cart_item['_accessoire_is_required'] == true) {
                        $parent_cart_item = WC()->cart->get_cart_item($cart_item['_accessoire_parent_key']);

                        if ($parent_cart_item) {
                            return (int)$parent_cart_item['quantity'];
                        } else {
                            // fallback if something went wrong and the accessoire product is still in the cart.
                            return 0;
                        }
                    }
                }
            }
            return $amount;
        }

        /**
         * Disable the remove button for required accessories.
         * @param $html
         * @param $cart_item_key
         */
        public function disableRemoveFromCartButton($html, $cart_item_key)
        {
            $Settings = UIRequiredAccessories::getSettings();

            if ($Settings['lock'] == 1) {
                $cart_item = WC()->cart->get_cart_item($cart_item_key);

                if (isset($cart_item['_accessoire_parent_key'])) {
                    if (isset($cart_item['_accessoire_is_required']) && $cart_item['_accessoire_is_required'] == true) {
                        return '';
                    }
                }
            }
            return $html;
        }

        /**
         * Add required products to the cart after adding the main product.
         */
        public function addToCart($cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data)
        {

            $AccessoireIDs = accessories::getAccessoireIDs($product_id, 'required');

            /*
             * Add all products to the cart
             */
            foreach ($AccessoireIDs as $key => $value) {
                WC()->cart->add_to_cart($value, $quantity, 0, array(), array(
                    '_accessoire_parent_key' => $cart_item_key,
                    '_accessoire_is_required' => true
                ));
            }
        }

        /**
         * Update the description with the settings
         * @param string $content
         * @param \WC_Product $product
         * @return mixed
         */
        public function addSettingsDescription($content, $product)
        {
            $description = $product->get_meta('f12-woo-required_accessoire_description', true);

            if (!$description || empty($description)) {

                $settings = UIRequiredAccessories::getSettings();
                if (!empty($settings['description'])) {
                    $content = $settings['description'];
                }
            } else {
                $content = $description;
            }
            return ReplacePlaceholder::replace($content, $product);
        }

        /**
         * Update the title with the settings
         * @param string $content
         * @param \WC_Product $product
         * @return mixed
         */
        public function addSettingsTitle($content, $product)
        {
            $title = $product->get_meta('f12-woo-required_accessoire_title', true);

            if (!$title || empty($title)) {
                $settings = UIRequiredAccessories::getSettings();
                if (!empty($settings['title'])) {
                    $content = $settings['title'];
                }
            } else {
                $content = $title;
            }
            return ReplacePlaceholder::replace($content, $product);
        }

        /**
         * Init the Frontend System
         */
        public function init()
        {
            $this->listShortcodes[] = new Shortcode();
        }

        /**
         * Return the accessories as string
         *
         * @param int The id of the post
         *
         * @return string
         */
        public static function getContent($postID)
        {
            $product = wc_get_product($postID);

            $accessoire_ids = accessories::getAccessoireIDs($product->get_id(), 'required');

            if (empty($accessoire_ids)) {
                return '';
            }

            $template = locate_template(
                array(
                    "woo-required-accessories.php",
                    "f12-woo-accesoires/woo-required-accessories.php"
                )
            );

            if (!$template) {
                $template = 'woo-required-accessories.php';
            }

            ob_start();
            include($template);
            $content = ob_get_contents();
            ob_end_clean();

            wp_reset_postdata();

            return $content;
        }

        /**
         * Render the Optional Accessories
         * @return string|void
         */
        public function render()
        {
            global $post;

            if (!$post) {
                return '';
            }

            echo self::getContent($post->ID);
        }

        /**
         * Shortcode to render the optional accessories
         */
        public function doShortcode($atts, $content = ''){
            $atts = shortcode_atts([
                'post_id' => '',
            ], $atts);

            if(empty($atts['post_id'])){
                return '';
            }

            return self::getContent($atts['post_id']);
        }
    }
}